package com.leagmain.flutter.photoloader

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.leagmain.android.photoloader.Photo
import com.leagmain.android.photoloader.PhotoJsonAdapter
import com.leagmain.android.photoloader.loadPhotos
import com.leagmain.android.photoloader.loadPhotosAsJsonString
import com.squareup.moshi.FromJson
import com.squareup.moshi.Moshi
import com.squareup.moshi.ToJson
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.util.*

/** PhotoLoaderPlugin */
class PhotoloaderPlugin : FlutterPlugin, MethodCallHandler {
    private lateinit var channel: MethodChannel

    private lateinit var applicationContext: Context

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        Log.d("Leon", "PhotoLoader plugin attached")
        applicationContext = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "photoloader")
        channel.setMethodCallHandler(this)
    }


    companion object {
        private const val ARG_ID = "id"
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "photoloader")
            channel.setMethodCallHandler(PhotoloaderPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "loadPhotosAsJsonString" -> {
                result.success(applicationContext.contentResolver.loadPhotosAsJsonString())
            }
            "loadPhotos" -> {
                val photos = applicationContext.contentResolver.loadPhotos()
                result.success(photos.map {
                    mapOf(
                            "id" to it.id,
                            "name" to it.displayName,
                            "path" to it.dataPath,
                            "title" to it.title
                    )
                })
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        Log.d("Leon", "PhotoLoader plugin attached")
        channel.setMethodCallHandler(null)
    }
}
